/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "obj_any.h"

#ifndef OSM_OBJ_WAY_H
#define OSM_OBJ_WAY_H

#include <genvector/vtp0.h>
#include "osm_typedefs.h"
#include "vtI0.h"
#include "tags.h"

typedef struct osm_way_data_s {
	vtI0_t nd;
	osm_tags_t tags;
} osm_way_data_t;

struct osm_way_s {
	osm_hdr_t hdr;
	osm_way_data_t *curr;
	osm_way_data_t orig;

	/* cache */
	vtp0_t ndp;
	long ndp_rev; /* vs. layer->rev */
};

void osm_way_init(osm_way_t *way);
osm_way_t *osm_way_new(void);

/*** utility ***/

/* Returns 1 if node is used by way */
int osm_way_refers_node(osm_way_t *way, osm_node_t *node);

/* List and mark ways that refer to node */
long osm_way_list_referers_by_node(vtp0_t *lst, osm_node_t *node);

/* Mark relations dirty if they refer node */
long osm_way_dirt_referers_by_node(osm_node_t *node);

/* Undoable node append to way's node list */
void osm_way_modify_append(osm_layer_t *layer, osm_way_t *way, osm_id_t node_id, int undoable);
void osm_way_modify_insert_before(osm_layer_t *layer, osm_way_t *way, osm_id_t node_id, long after_nd_idx, int undoable);

void osm_way_modify_fork(osm_layer_t *layer, osm_way_t *way, int undoable);

void osm_way_modify_remove(osm_way_t *way, long idx, int undoable);

void osm_way_modify_reverse(osm_way_t *way, int undoable);

void osm_way_modify_replace(osm_way_t *way, long idx, osm_id_t new_node_id, int undoable);

osm_node_t *osm_way_modify_insert_near(osm_way_t *way, rnd_coord_t px, rnd_coord_t py, rnd_coord_t maxdist2, int undoable);

/* Split way at node into two ways; tags are copied from to the new way, new
   way is returned; node will be a shared endpoint */
osm_way_t *osm_way_modify_split_at(osm_way_t *way, osm_node_t *node, int dup_node, int undoable);
osm_way_t *osm_way_modify_split_near(osm_way_t *way, rnd_coord_t px, rnd_coord_t py, rnd_coord_t maxdist2, int dup_node, int undoable);


/* Re-generate the ndp cache if the layer has changed since last generated */
void osm_way_ensure_ndp(osm_way_t *way);

/* Find the and return the index of the node after which px;py
   should be inserted, precisely at *best_x_out and *best_y_out; return -1
   when not found */
long osm_way_find_near(osm_way_t *way, rnd_coord_t px, rnd_coord_t py, rnd_coord_t maxdist2, rnd_coord_t *best_x_out, rnd_coord_t *best_y_out);


void osm_way_pre_edit(osm_any_obj_t *o, int geo, int auto_fork);
void osm_way_post_edit(osm_any_obj_t *o, int tags_changed);


#endif
