/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - Bing map tiles
 *  Copyright (C) 2023,2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include <ctype.h>
#include <genht/htsp.h>
#include <genht/hash.h>
#include <genvector/gds_char.h>
#include <librnd/core/compat_fs.h>
#include <librnd/core/safe_fs.h>
#include <librnd/core/paths.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "conf_core.h"

static gds_t tms_tmp;


TODO("these should be configurable")
#define CACHE_DIR  "~/" DOT_OSM_RND "/cache/tms"

static const char *tms_build_url_for(rnd_design_t *dsg, osm_tilecache_t *ctx, const char *key, void *tileaddr_)
{
	tms_t *tms = ctx->user_data;
	tms_tileaddr_t *tileaddr = tileaddr_;
	tms_url_seg_t *seg;

	tms_tmp.used = 0;
	for(seg = tms->url_segs; seg != NULL; seg = seg->next) {
		switch(seg->type) {
			case TUS_STR:  gds_append_str(&tms_tmp, seg->data.str); break;
			case TUS_X:    rnd_append_printf(&tms_tmp, "%ld", tileaddr->x); break;
			case TUS_Y:    rnd_append_printf(&tms_tmp, "%ld", tileaddr->y); break;
			case TUS_Z:    rnd_append_printf(&tms_tmp, "%d", tileaddr->z); break;
			case TUS_SWITCH:
				seg->cnt++;
				gds_append_str(&tms_tmp, seg->data.sw.array[seg->cnt % seg->data.sw.used]); break;
				break;
		}
	}

	return tms_tmp.array;
}

static void tms_snprint_disk_name(rnd_design_t *dsg, osm_tilecache_t *ctx, const char *key, void *tileaddr, char *dst, size_t max_len)
{
	rnd_snprintf(ctx->cache_disk_path_end, max_len, "%s.jpg", key);
}


static void tms_cache_init(rnd_design_t *dsg, tms_t *tms)
{
	if (tms->cache.inited == 0) {
		char *curr, *next;
		long namelen = strlen(tms->name);

		osm_tilecache_init(&tms->cache);

		tms->cache.build_url_for = tms_build_url_for;
		tms->cache.snprint_disk_name = tms_snprint_disk_name;
		tms->cache.user_data = tms;

		rnd_path_resolve(dsg, CACHE_DIR, &tms->cache.cache_disk_path, 256+namelen, 0);
		for(curr = tms->cache.cache_disk_path; curr != NULL; curr = next) {
			next = strchr(curr, '/');
			if (next != NULL)
				*next = '\0';
			rnd_mkdir(dsg, tms->cache.cache_disk_path, 0755);
			if (next != NULL) {
				*next = '/';
				next++;
			}
		}
		tms->cache.cache_disk_path_end = tms->cache.cache_disk_path + strlen(tms->cache.cache_disk_path);
		*tms->cache.cache_disk_path_end = '/';
		tms->cache.cache_disk_path_end++;

		/* cache is put in a separate dir per layer name */
		strcpy(tms->cache.cache_disk_path_end, tms->name);
		rnd_mkdir(dsg, tms->cache.cache_disk_path, 0755);
		tms->cache.cache_disk_path_end += namelen;

		*tms->cache.cache_disk_path_end = '/';
		tms->cache.cache_disk_path_end++;
	}
	tms->cache.inited++;
}

static rnd_pixmap_t *tms_cache_get(rnd_design_t *dsg, tms_t *tms, const char *key, void *tileaddr, time_t now)
{
	return osm_tilecache_get(dsg, &tms->cache, key, tileaddr, now);
}

