/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - Presets for tagging
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include <librnd/core/safe_fs.h>

lht_doc_t *presets = NULL;
vtp0_t presets_sorted;

TODO("This should be coming from the config")
#define PRESETS_DB "presets.lht"

static void presets_unload(void)
{
	if (presets != NULL) {
		lht_dom_uninit(presets);
		presets = NULL;
	}
	vtp0_uninit(&presets_sorted);
}

static int pld_cmp(const void *a, const void *b)
{
	lht_node_t *const *na = a, *const *nb = b;
	return strcmp((*na)->name, (*nb)->name);
}

static void presets_load(void)
{
	FILE *f;
	char *errmsg;
	lht_node_t *parent, *n;
	lht_dom_iterator_t it;

	if (presets != NULL)
		presets_unload();

	f = rnd_fopen(NULL, PRESETS_DB, "r");
	if (f == NULL) {
		rnd_message(RND_MSG_ERROR, "Failed to open presets database %s\n", PRESETS_DB);
		return;
	}

	presets = lht_dom_load_stream(f, PRESETS_DB, &errmsg);
	if (presets == NULL) {
		rnd_message(RND_MSG_ERROR, "Failed to load presets database from %s:\n", PRESETS_DB);
		rnd_message(RND_MSG_ERROR, errmsg);
		rnd_message(RND_MSG_ERROR, "\n");
	}
	fclose(f);

	/* sort presets */
	parent = lht_dom_hash_get(presets->root, "presets");
	for(n = lht_dom_first(&it, parent); n != NULL; n = lht_dom_next(&it))
		vtp0_append(&presets_sorted, n);
	if (presets_sorted.used > 0)
		qsort(presets_sorted.array, presets_sorted.used, sizeof(lht_node_t *), pld_cmp);
}

