/*
 *                            COPYRIGHT
 *
 *  osm-rnd - OpenStreetMap editor/viewer - standard tools
 *  Copyright (C) 2024 Tibor 'Igor2' Palinkas
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Contact:
 *    Project page: http://repo.hu/projects/osm-rnd
 *    lead developer: http://repo.hu/projects/osm-rnd/contact.html
 *    mailing list: (not yet)
 */

#include "config.h"

#include <librnd/hid/tool.h>

#include "data.h"
#include "search.h"
#include "crosshair.h"


void osm_tool_split_press(rnd_design_t *hl)
{
	osm_design_t *osm = (osm_design_t *)hl;
	osm_layer_t *ly = osm_layer_get_curr(osm);
	osm_any_obj_t *obj;

	if (ly == NULL) {
		rnd_message(RND_MSG_ERROR, "Select a layer first!\n");
		return;
	}

	if (!ly->editable) {
		rnd_message(RND_MSG_ERROR, "Can't split object: layer is not editable\n");
		return;
	}

	obj = osm_search_obj_xy_mask(ly, osm->crosshair_x, osm->crosshair_y, OSM_MASK_NODE | OSM_MASK_WAY);
	if (obj == NULL)
		return;

	if (obj->hdr.type == OSM_OBJ_NODE) {
		vtp0_t lst = {0};
		long n;
		int dup_node = !rnd_gui->shift_is_pressed(rnd_gui);

		osm_obj_list_referers(&lst, obj, OSM_MASK_WAY);

		uundo_freeze_serial(&ly->undo);

		for(n = 0; n < lst.used; n++)
			osm_way_modify_split_at(lst.array[n], &obj->node, dup_node, 1);

		uundo_unfreeze_serial(&ly->undo);
		uundo_inc_serial(&ly->undo);

		vtp0_uninit(&lst);
	}
	else if (obj->hdr.type == OSM_OBJ_WAY) {
		rnd_coord_t px = osm->crosshair_x, py = osm->crosshair_y, slp2 = osm_slop * osm_slop;
		int dup_node = !rnd_gui->shift_is_pressed(rnd_gui);

		osm_way_modify_split_near(&obj->way, px, py, slp2, dup_node, 1);
	}
}

/* XPM */
static const char *split_icon[] = {
/* columns rows colors chars-per-pixel */
"21 21 3 1",
"  c black",
". c gray100",
"X c None",
/* pixels */
"XXXXX  XXXXXXX  XXXXX",
"XXXXXX  XXXXX  XXXXXX",
"XXXXXXX  XXX  XXXXXXX",
"XXXXXXXX  X  XXXXXXXX",
"XXXXXXXXX   XXXXXXXXX",
"XXXXXXX  XXX  XXXXXXX",
"XXXXXX  XXXXX  XXXXXX",
"XX..  .XXXXXXX.  ..XX",
"X.    .XXXXXXX.    .X",
"X.    .XXXXXXX.    .X",
"XX.  .XXXXXXXXX.  .XX",
"XXX  XXXXXXXXXXX  XXX",
"XXXXXXXXXXXXXXXXXXXXX",
"XX   XX    XX X     X",
"X XXX X XXX X XXX XXX",
"X XXXXX XXX X XXX XXX",
"XX XXXX XXX X XXX XXX",
"XXX XXX    XX XXX XXX",
"XXXX XX XXXXX XXX XXX",
"X XXX X XXXXX XXX XXX",
"XX   XX XXXXX   X XXX"
};

rnd_tool_t osm_rnd_tool_split = {
	"split", NULL, NULL, 100, split_icon, RND_TOOL_CURSOR_NAMED("up_arrow"), 0,
	NULL,
	NULL,
	osm_tool_split_press,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL /* escape */
};
